﻿define("ace/mode/logofr_highlight_rules",["require","exports","module","ace/lib/oop","ace/mode/text_highlight_rules"], function(require, exports, module) {
"use strict";

var oop = require("../lib/oop");
var TextHighlightRules = require("./text_highlight_rules").TextHighlightRules;

var LogoFRpHighlightRules = function() {


    var keywordMapper = this.createKeywordMapper({
        "variable.language": "",
        "keyword": 
            " ARRONDI ATTENDS AVANCE AV " +
            " BAISSECRAYON BC BLANC BLEU BRUN"+
            " CACHETORTUE CAP CHOIX CHOSE COMPTE COMPTEUR.R COS CTO CYAN " +
            " DERNIER DESQUE DIFFERENCE DONNE DONNELOCALE DR DROITE " +
            " EGAL? ENLEVE ENT ET ETIQUETTE EXEC EXECUTE " +
            " FAUX FCC FTC FIXECAP FIXECOULEURCRAYON FIXEPOS FIXETAILLECRAYON FIXEX FIXEXY FIXEY " +
            " GA GAUCHE GRIS" +
            " HASARD " +
            " INVERSE ITEM " +
            " JAUNE " +
            " " +
            " LC LEVECRAYON LISTE LISTE? LOCALE LOG10 " +
            " MAGENTA MELANGE MD METSDERNIER METSPREMIER MOINS MONTRE MONTRETORTUE MOT MOT? MP MTO " +
            " NETTOIE NETTOIETOUT NOIR NOMBRE? NON NT " +
            " ORIGINE OU "+
            " PI PHRASE PLUSGRAND? PLUSPETIT? POS PREMIER PRODUIT PUISSANCE "+
            " QUOTIENT " +
            " RACINE RAPPORTE RE RECULE REMPLIS REPETE REPETEPOUR RESTE RENDS RETOURNE ROUGE " +
            " SAUFDERNIER SAUFPREMIER SD SI SIN SINON SP SOMME STOP STOPPE " +
            " TANTQUE TD TG TRONQUE "+
            " " +
            " VE VERT VIDE? VIDEECRAN VRAI" +
            " " +
            " ",
        "constant.language": 
            "POUR FIN VRAI FAUX",
        "support.type": 
            "",
        "keyword.operator":
            "ARRONDI COS DIFFERENCE LOG10 MOINS PI PUISSANCE QUOTIENT RACINE SIN "
    }, "text", true, " ");

    var compoundKeywords = "WITH\\W+(?:HEADER\\W+LINE|FRAME|KEY)|NO\\W+STANDARD\\W+PAGE\\W+HEADING|"+
        "EXIT\\W+FROM\\W+STEP\\W+LOOP|BEGIN\\W+OF\\W+(?:BLOCK|LINE)|BEGIN\\W+OF|"+
        "END\\W+OF\\W+(?:BLOCK|LINE)|END\\W+OF|NO\\W+INTERVALS|"+
        "RESPECTING\\W+BLANKS|SEPARATED\\W+BY|USING\\W+(?:EDIT\\W+MASK)|"+
        "WHERE\\W+(?:LINE)|RADIOBUTTON\\W+GROUP|REF\\W+TO|"+
        "(?:PUBLIC|PRIVATE|PROTECTED)(?:\\W+SECTION)?|DELETING\\W+(?:TRAILING|LEADING)"+
        "(?:ALL\\W+OCCURRENCES)|(?:FIRST|LAST)\\W+OCCURRENCE|INHERITING\\W+FROM|"+
        "LINE-COUNT|ADD-CORRESPONDING|AUTHORITY-CHECK|BREAK-POINT|CLASS-DATA|CLASS-METHODS|"+
        "CLASS-METHOD|DIVIDE-CORRESPONDING|EDITOR-CALL|END-OF-DEFINITION|END-OF-PAGE|END-OF-SELECTION|"+
        "FIELD-GROUPS|FIELD-SYMBOLS|FUNCTION-POOL|MOVE-CORRESPONDING|MULTIPLY-CORRESPONDING|NEW-LINE|"+
        "NEW-PAGE|NEW-SECTION|PRINT-CONTROL|RP-PROVIDE-FROM-LAST|SELECT-OPTIONS|SELECTION-SCREEN|"+
        "START-OF-SELECTION|SUBTRACT-CORRESPONDING|SYNTAX-CHECK|SYNTAX-TRACE|TOP-OF-PAGE|TYPE-POOL|"+
        "TYPE-POOLS|LINE-SIZE|LINE-COUNT|MESSAGE-ID|DISPLAY-MODE|READ(?:-ONLY)?|"+
        "IS\\W+(?:NOT\\W+)?(?:ASSIGNED|BOUND|INITIAL|SUPPLIED)";
     
    this.$rules = {
        "start" : [
            {
                token : "comment",
                regex : "[;|#]",
                next : "singleLineComment"
            },            
            {
                token : "comment", // multi line comment
                regex : "\\/\\*",
                next : "comment"
            },
            {token : "constant.language.boolean", regex : /(?:VRAI|FAUX)\b/},
            {token : "constant.numeric", regex: "[+-]?\\d+\\b"},
            {token : "constant.string", regex: "\"\\w+\\b"},
            {token : "variable.parameter", regex : "\:\\w*"},
            {token : keywordMapper, regex : "\\b\\w+\\.?\\w+\\b\\??"},
            {caseInsensitive: true}
        ],
        "comment" : [
            {
                token : "comment", // closing comment
                regex : ".*?\\*\\/",
                next : "start"
            }, {
                token : "comment", // comment spanning whole line
                regex : ".+"
            }
        ],
        "singleLineComment" : [
            {
                token : "comment",
                regex : /\\$/,
                next : "singleLineComment"
            }, {
                token : "comment",
                regex : /$/,
                next : "start"
            }, {
                defaultToken: "comment"
            }
        ]
    };
};
oop.inherits(LogoFRpHighlightRules, TextHighlightRules);

exports.LogoFRpHighlightRules = LogoFRpHighlightRules;
});


define("ace/mode/folding/coffee",["require","exports","module","ace/lib/oop","ace/mode/folding/fold_mode","ace/range"], function(require, exports, module) {
"use strict";

var oop = require("../../lib/oop");
var BaseFoldMode = require("./fold_mode").FoldMode;
var Range = require("../../range").Range;

var FoldMode = exports.FoldMode = function() {};
oop.inherits(FoldMode, BaseFoldMode);

(function() {

    this.getFoldWidgetRange = function(session, foldStyle, row) {
        var range = this.indentationBlock(session, row);
        if (range)
            return range;

        var re = /\S/;
        var line = session.getLine(row);
        var startLevel = line.search(re);
        if (startLevel == -1 || line[startLevel] != "#")
            return;

        var startColumn = line.length;
        var maxRow = session.getLength();
        var startRow = row;
        var endRow = row;

        while (++row < maxRow) {
            line = session.getLine(row);
            var level = line.search(re);

            if (level == -1)
                continue;

            if (line[level] != "#")
                break;

            endRow = row;
        }

        if (endRow > startRow) {
            var endColumn = session.getLine(endRow).length;
            return new Range(startRow, startColumn, endRow, endColumn);
        }
    };
    this.getFoldWidget = function(session, foldStyle, row) {
        var line = session.getLine(row);
        var indent = line.search(/\S/);
        var next = session.getLine(row + 1);
        var prev = session.getLine(row - 1);
        var prevIndent = prev.search(/\S/);
        var nextIndent = next.search(/\S/);

        if (indent == -1) {
            session.foldWidgets[row - 1] = prevIndent!= -1 && prevIndent < nextIndent ? "start" : "";
            return "";
        }
        if (prevIndent == -1) {
            if (indent == nextIndent && line[indent] == "#" && next[indent] == "#") {
                session.foldWidgets[row - 1] = "";
                session.foldWidgets[row + 1] = "";
                return "start";
            }
        } else if (prevIndent == indent && line[indent] == "#" && prev[indent] == "#") {
            if (session.getLine(row - 2).search(/\S/) == -1) {
                session.foldWidgets[row - 1] = "start";
                session.foldWidgets[row + 1] = "";
                return "";
            }
        }

        if (prevIndent!= -1 && prevIndent < indent)
            session.foldWidgets[row - 1] = "start";
        else
            session.foldWidgets[row - 1] = "";

        if (indent < nextIndent)
            return "start";
        else
            return "";
    };

}).call(FoldMode.prototype);

});

define("ace/mode/logofr",["require","exports","module","ace/mode/logofr_highlight_rules","ace/mode/folding/coffee","ace/range","ace/mode/text","ace/lib/oop"], function(require, exports, module) {
"use strict";

var Rules = require("./logofr_highlight_rules").LogoFRpHighlightRules;
var FoldMode = require("./folding/coffee").FoldMode;
var Range = require("../range").Range;
var TextMode = require("./text").Mode;
var oop = require("../lib/oop");

function Mode() {
    this.HighlightRules = Rules;
    this.foldingRules = new FoldMode();
}

oop.inherits(Mode, TextMode);

(function() {
    
    this.getNextLineIndent = function(state, line, tab) {
        var indent = this.$getIndent(line);
        return indent;
    };
    
    this.toggleCommentLines = function(state, doc, startRow, endRow){
        var range = new Range(0, 0, 0, 0);
        for (var i = startRow; i <= endRow; ++i) {
            var line = doc.getLine(i);
            if (hereComment.test(line))
                continue;
                
            if (commentLine.test(line))
                line = line.replace(commentLine, '$1');
            else
                line = line.replace(indentation, '$&#');
    
            range.end.row = range.start.row = i;
            range.end.column = line.length + 1;
            doc.replace(range, line);
        }
    };
    
    this.$id = "ace/mode/logofr";
}).call(Mode.prototype);

exports.Mode = Mode;

});
