/* **************** ***********************************************************/
/* Monde  ********************************************************************/
/* JSLing 20150630                                                           */
/* ***************************************************************************/
"use strict";

function Monde(logo, nom_canvas) { /******************************************/
    this.canvas = document.getElementById(nom_canvas);
    this.murs = [];
    this.LWLogo = logo;
    this.quadrillage = 6;
    if (this.canvas) {
        this.largeur = this.canvas.width;
        this.hauteur = this.canvas.height;
        this.draw();
    }
} // Monde


Monde.prototype.collision = function (aab) { /********************************/
    var k, l, o;
    if (aab) {
        if ((aab.x < 0) || (aab.x1 > this.largeur) || (aab.y < 0) || (aab.y1 > this.hauteur)) {
            return true;
        }
        k = this.murs.length;
        for (l = 0; l < k; l++) {
            o = this.murs[l].aab;
            if (this.collisionAABB(o, aab)) { return true; }
        }
    }
    return false;
}; // Monde.collision

// Collision entre un point et une Axis Aligned Bounding Box /****************/
Monde.prototype.collisionXY = function (x, y, box) { /************************/
    if ((x >= box.x) && (x < box.x + box.w) && (y >= box.y) && (y < box.y + box.h)) {
        return true;
    }
    return false;
}; // Monde.collisionXY

// Collision entre deux Axis Aligned Bounding Box /***************************/
Monde.prototype.collisionAABB = function (box1, box2) {
    if ((box2.x >= box1.x + box1.w) || (box2.x + box2.w <= box1.x) || (box2.y >= box1.y + box1.h) || (box2.y + box2.h <= box1.y)) {
        return false;
    }
    return true;
}; // Monde.collisionAABB

// Collision entre un point et un cercle /************************************/
Monde.prototype.collisionXYC = function (x, y, cercle) { /********************/
    var d2 = (x - cercle.x) * (x - cercle.x) + (y - cercle.y) * (y - cercle.y);
    if (d2 > cercle.rayon * cercle.rayon) {
        return false;
    }
    return true;
}; // Monde.collisionXYC

// Collision entre 2 cercles /************************************************/
Monde.prototype.collisionCC = function (c1, c2) { /***************************/
    var d2 = (c1.x - c2.x) * (c1.x - c2.x) + (c1.y - c2.y) * (c1.y - c2.y);
    if (d2 > (c1.rayon + c2.rayon) * (c1.rayon + c2.rayon)) {
        return false;
    }
    return true;
}; // Monde.collisionCC

Monde.prototype.draw = function () { /****************************************/
    var i, j,
    ctx = this.canvas.getContext("2d");
    ctx.fillStyle = "rgba(255,255,255,0.1)";
    ctx.clearRect(0, 0, this.largeur, this.hauteur);
    ctx.fillRect(0, 0, this.largeur, this.hauteur);

    switch (this.quadrillage) {
	case 0  :    break; // aucun
	case 1 :    ctx.lineWidth = 1; // Papier Point 10x10
                ctx.beginPath();
                ctx.strokeStyle="rgba(255, 0, 0, 1)";
                i = this.largeur / 2;
                ctx.moveTo(i,0);ctx.lineTo(i, this.hauteur);
                i = this.hauteur / 2;
                ctx.moveTo(0, i); ctx.lineTo(this.largeur, i);
                ctx.stroke();
                i -= 10; j=this.largeur/2;
                ctx.beginPath();
                ctx.textAlign = "center";
                ctx.strokeStyle = "rgba(0, 0, 128, 0.5)";
                ctx.fillStyle = "#000000";
                i=0;j=0;
		while (j<this.hauteur) {
			while (i <this.largeur) {
				ctx.moveTo(i-1,j);ctx.lineTo(i+1, j);
				ctx.moveTo(i,j-1);ctx.lineTo(i ,j+1);
				i += 10;
			}
			j+=10;
			i=0;
		}
                ctx.stroke();
                break;
	case 2 :    ctx.lineWidth = 1; // Papier Point 20x20
                ctx.beginPath();
                ctx.strokeStyle="rgba(255, 0, 0, 1)";
                i = this.largeur / 2;
                ctx.moveTo(i,0);ctx.lineTo(i, this.hauteur);
                i = this.hauteur / 2;
                ctx.moveTo(0, i); ctx.lineTo(this.largeur, i);
                ctx.stroke();
                i -= 20; j=this.largeur/2;
                ctx.beginPath();
                ctx.textAlign = "center";
                ctx.strokeStyle = "rgba(0, 0, 128, 0.5)";
                ctx.fillStyle = "#000000";
                i=0;j=0;
		while (j<this.hauteur) {
			while (i <this.largeur) {
				ctx.moveTo(i-1,j);ctx.lineTo(i+1, j);
				ctx.moveTo(i,j-1);ctx.lineTo(i ,j+1);
				i += 20;
			}
			j+=20;
			i=0;
		}
                ctx.stroke();
                break;
	case 3 :    ctx.lineWidth = 1; // Papier Point 50x50
                ctx.beginPath(); // Dbut du trac
                ctx.strokeStyle="rgba(255, 0, 0, 1)"; //rouge
                i = this.largeur / 2;j = this.hauteur / 2; // Centrage du curseur
                // Dessin du point central en rouge
		ctx.moveTo(i-2,this.hauteur/2);ctx.lineTo(i+2, this.hauteur/2);
                ctx.moveTo(this.largeur/2, j-2); ctx.lineTo(this.largeur/2, j+2);
                ctx.stroke(); // affichage du point
		// Dbut du trac des autres points
                ctx.beginPath();
                ctx.strokeStyle = "rgba(0, 0, 128, 0.5)"; // Gris fonc
                ctx.fillStyle = "#000000"; // Style de trac
                i=0;j=0;// Dpart en haut  gauche
		while (j<this.hauteur) { // Boucle sur la hauteur
			while (i <this.largeur) {// Boucle sur la largeur
				ctx.moveTo(i-1,j);ctx.lineTo(i+1, j); // ligne horizontale
				ctx.moveTo(i,j-1);ctx.lineTo(i ,j+1); // ligne verticale
				i += 50; // colonne suivante
			}
			j+=50; // Ligne suivante
			i=0; // Retour  la 1re colonne
		}
                ctx.stroke(); // Affichage du trac
                break;
	case 4 :    ctx.lineWidth = 1; // Petits Carreaux avec Reperes 10x10
                ctx.beginPath();
                ctx.strokeStyle="rgba(255, 0, 0, 1)";
                i = this.largeur / 2;
                ctx.moveTo(i,0);ctx.lineTo(i, this.hauteur);
                i = this.hauteur / 2;
                ctx.moveTo(0, i); ctx.lineTo(this.largeur, i);
                ctx.stroke();
                i = this.largeur / 2; j = i; i -= 10;
                ctx.beginPath();
                ctx.textAlign = "center";
                ctx.strokeStyle = "rgba(0, 0, 128, 0.5)";
                ctx.fillStyle = "#000000";
                while (i > 0) {
                    if ((i % 50)==0) {ctx.fillText(j - i, i, this.hauteur - 5)};
                    ctx.moveTo(i, 0);ctx.lineTo(i, this.hauteur);
                    i -= 10;
                }
                i = this.largeur / 2; j = i; i += 10;
                while (i<this.largeur) {
                    if ((i % 50)==0) {ctx.fillText(i - j, i, this.hauteur - 5)};
                    ctx.moveTo(i, 0);ctx.lineTo(i, this.hauteur);
                    i += 10;
                }
                i = this.hauteur / 2; j = i; i -= 10;
                while (i > 0) {
                    if ((i % 50)==0) {ctx.fillText(j - i,this.largeur - 15,i + 5)};
                    ctx.moveTo(0, i);ctx.lineTo(this.largeur, i);
                    i -= 10;
                }
                i = this.hauteur / 2; j = i; i += 10;
                while (i<this.hauteur) {
                    if ((i % 50)==0) {ctx.fillText(i-j,this.largeur - 15,i + 5)};
                    ctx.moveTo(0, i);ctx.lineTo(this.largeur, i);
                    i += 10;
                }
                ctx.stroke();
                break;
	case 5  :  ctx.lineWidth = 1; // Reperes 20x20
                ctx.beginPath();
                ctx.strokeStyle="rgba(255, 0, 0, 1)";
                i = this.largeur / 2;
                ctx.moveTo(i,0);ctx.lineTo(i, this.hauteur);
                i = this.hauteur / 2;
                ctx.moveTo(0, i); ctx.lineTo(this.largeur, i);
                ctx.stroke();
                i = this.largeur / 2; j = i; i -= 20;
                ctx.beginPath();
                ctx.textAlign = "center";
                ctx.strokeStyle = "rgba(0, 0, 128, 0.5)";
                ctx.fillStyle = "#000000";
                while (i > 0) {
                    ctx.fillText(j - i, i, this.hauteur - 5);
                    ctx.moveTo(i, 0);ctx.lineTo(i, this.hauteur);
                    i -= 20;
                }
                i = this.largeur / 2; j = i; i += 20;
                while (i<this.largeur) {
                    ctx.fillText(i - j, i, this.hauteur - 5);
                    ctx.moveTo(i, 0);ctx.lineTo(i, this.hauteur);
                    i += 20;
                }
                i = this.hauteur / 2; j = i; i -= 20;
                while (i > 0) {
                    ctx.fillText(j - i,this.largeur - 15,i + 5);
                    ctx.moveTo(0, i);ctx.lineTo(this.largeur, i);
                    i -= 20;
                }
                i = this.hauteur / 2; j = i; i += 20;
                while (i<this.hauteur) {
                    ctx.fillText(i-j,this.largeur - 15,i + 5);
                    ctx.moveTo(0, i);ctx.lineTo(this.largeur, i);
                    i += 20;
                }
                ctx.stroke();
                break;
	case 6 :    ctx.lineWidth = 1; // Reperes 50x50
                ctx.beginPath();
                ctx.strokeStyle="rgba(255, 0, 0, 1)";
                i = this.largeur / 2;
                ctx.moveTo(i,0);ctx.lineTo(i, this.hauteur);
                i = this.hauteur / 2;
                ctx.moveTo(0, i); ctx.lineTo(this.largeur, i);
                ctx.stroke();
                i = this.largeur / 2; j = i; i -= 50;
                ctx.beginPath();
                ctx.textAlign = "center";
                ctx.strokeStyle = "rgba(0, 0, 128, 0.5)";
                ctx.fillStyle = "#000000";
                while (i > 0) {
                    ctx.fillText(j - i, i, this.hauteur - 5);
                    ctx.moveTo(i, 0);ctx.lineTo(i, this.hauteur);
                    i -= 50;
                }
                i = this.largeur / 2; j = i; i += 50;
                while (i<this.largeur) {
                    ctx.fillText(i - j, i, this.hauteur - 5);
                    ctx.moveTo(i, 0);ctx.lineTo(i, this.hauteur);
                    i += 50;
                }
                i = this.hauteur / 2; j = i; i -= 50;
                while (i > 0) {
                    ctx.fillText(j - i,this.largeur - 15,i + 5);
                    ctx.moveTo(0, i);ctx.lineTo(this.largeur, i);
                    i -= 50;
                }
                i = this.hauteur / 2; j = i; i += 50;
                while (i<this.hauteur) {
                    ctx.fillText(i-j,this.largeur - 15,i + 5);
                    ctx.moveTo(0, i);ctx.lineTo(this.largeur, i);
                    i += 50;
                }
                ctx.stroke();
                break;
	case 7 :    ctx.lineWidth = 1; // Seyes;
                // Lignes Verticales
		ctx.beginPath();
                ctx.strokeStyle="rgba(74, 0, 181, 1)"; // Mauve
                for (i = 0; i < this.largeur; i += 40) {
                     ctx.moveTo(i, 0);
                     ctx.lineTo(i, this.hauteur);
                }
		ctx.stroke();
		// Lignes Horizontales
                ctx.beginPath();
                ctx.strokeStyle = "rgba(129, 0, 131, 1)"; // Violet
                for (i = 10; i < this.largeur; i += 40) {
                     ctx.moveTo(0, i);
                     ctx.lineTo(this.largeur, i);
                }
                ctx.stroke();
		// Interlignes
                ctx.beginPath();
                ctx.strokeStyle = "rgba(154, 205, 248, 1)"; // Bleu ple
                for (i = 0; i < this.largeur; i += 10) {
                    if (i % 40!== 10) { ctx.moveTo(0, i); ctx.lineTo(this.largeur, i); }
                }
                ctx.stroke();
                break;
	default: // Blanc;
	}
    for (i = 0; i < this.murs.length; i++) {
        ctx.fillStyle = "#663333";
        ctx.strokeStyle = "#000000";
        ctx.fillRect(this.murs[i].aab.x, this.murs[i].aab.y, this.murs[i].w, this.murs[i].h);
    }
}; // draw

Monde.prototype.mur = function (p) { /****************************************/
    var o;
    if ((p.length > 3) && (Math.abs(p[2]) > 0) && (Math.abs(p[3]) > 0)) {
        if ((p[0] < this.largeur) && (p[1] < this.hauteur)) {
            o = new ObjetStatique('M', p[0], p[1], p[2], p[3], this);
            this.murs.push(o);
        }
    }
    this.draw();
    if (this.LWLogo.troisD) {
            this.LWLogo.troisD.maj_monde();
    }
}; // Monde.mur

Monde.prototype.reset = function() { /****************************************/
    this.murs=[];
    this.draw();
    if (this.LWLogo.troisD) {
            this.LWLogo.troisD.maj_monde();
    }
}; // Monde.reset

/****************************/

function ObjetStatique(type, x, y, w, h, monde) { /***************************/
    this.x = x;
    this.y = y;
    this.w = w;
    this.h = h;
    this.aab = new AAB(this.x + (monde.largeur / 2),(monde.hauteur/2) - this.y, w, h) ;
} // ObjetStatique

